package framework.everywaretechnologies.it.icarus;

import org.json.JSONException;
import org.json.JSONObject;

public class EventTrackResource extends IcarusResource {

    public enum EventCategories{ //categorie di evento

        APPLICATION,ACTION,NOTIFICATION;
    }

    public enum EventLabel{

        FOREGROUND,BACKGROUND;
    }

    private String eventName;
    private String eventCategory;
    private String eventLabel;

    private int heading;
    private int course;
    private int speed;

    //i due successivi attributi sono quelli ritornati dall'ActivityRecognition di Google che dev'essere implementata nell'app chiamante
    //convertiti da quelli di Google a quelli del server.. vedi iMoveAndroid per il metodo di mapping tra i valori ritornati da Google e quelli
    //per il server di ewtech. Fare copia incolla del metodo in progetti futuri. Non lo metto qua perchè non ho accesso ai codici delle attività
    //non avendo la dipendenza all'ActivityRecognition all'interno di questa libreria
    private String mostProbableUserActivityName;
    private int confidence;

    private boolean accessibilityEnabled;

    private String[] userActivityToTrack;

    public EventTrackResource(String _eventName, String _eventCategory, String _eventLabel, int _heading, int _course, int _speed,
                              String _mostProbableUserActivityName, int _confidence, boolean _accessibilityEnabled){

        super();

        eventName=_eventName;
        eventCategory=_eventCategory;
        eventLabel=_eventLabel;
        heading=_heading;
        course=_course;
        speed=_speed;
        mostProbableUserActivityName=_mostProbableUserActivityName;
        confidence=_confidence;
        accessibilityEnabled=_accessibilityEnabled;

        userActivityToTrack=new String[]{"stationary","walking","running","automotive","cycling","unknown"};
    }

    public JSONObject getEventTrackResourceInJSONFormat(){

        JSONObject userActivityJSONObject=new JSONObject();

        try{

            for(String activityType: userActivityToTrack){

                if(activityType.equalsIgnoreCase(mostProbableUserActivityName))
                    userActivityJSONObject.put(activityType,true);
                else
                    userActivityJSONObject.put(activityType,false);

            }
            userActivityJSONObject.put("confidence",confidence);
            userActivityJSONObject.put("startTimestamp",System.currentTimeMillis());

        } catch (JSONException jsonException){

            jsonException.printStackTrace();
        }

        JSONObject directionJSONObject=new JSONObject();

        try {

            directionJSONObject.put("heading",heading); //0..360
            directionJSONObject.put("course",course); //0..360

        } catch (JSONException jsonException){

            jsonException.printStackTrace();
        }

        JSONObject eventContextJSONObject=new JSONObject();

        try {

            eventContextJSONObject.put("speed",speed);
            eventContextJSONObject.put("activity",userActivityJSONObject);
            eventContextJSONObject.put("direction",directionJSONObject);

        } catch (JSONException jsonException){

            jsonException.printStackTrace();
        }

        JSONObject toSend=new JSONObject();

        try {

            toSend.put("EventName",eventName);
            toSend.put("EventCategory",eventCategory);
            toSend.put("EventLabel",eventLabel+"_TalkBack"+(accessibilityEnabled?"_ON":"_OFF"));
            toSend.put("EventContext",eventContextJSONObject);

        } catch (JSONException jsonException){

            jsonException.printStackTrace();
        }

        return toSend;
    }


}
